var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { jsx as _jsx, Fragment as _Fragment, jsxs as _jsxs } from "react/jsx-runtime";
import { Box, Button, IconButton, Modal, ModalContent, ModalOverlay, useColorMode, Editable, EditableInput, EditablePreview, Popover, PopoverTrigger, PopoverContent, Divider, Portal, } from '@chakra-ui/react';
import { useDeep, useDeepSubscription } from '@deep-foundation/deeplinks/imports/client';
import { useMinilinksFilter } from '@deep-foundation/deeplinks/imports/minilinks';
import { useLocalStore } from '@deep-foundation/store/local';
import { useDebounceCallback } from '@react-hook/debounce';
import json5 from 'json5';
import dynamic from 'next/dynamic';
import React, { useCallback, useEffect, useMemo, useRef, useState } from 'react';
import { VscClearAll } from 'react-icons/vsc';
import { ClientHandler, ClientHandlerRenderer, evalClientHandler } from '../client-handler';
import { EditorComponentView } from '../editor/editor-component-view';
import { EditorGrid } from '../editor/editor-grid';
import { ListLanguages } from '../editor/editor-lang-list';
import { EditorResults } from '../editor/editor-results';
import { EditorSwitcher } from '../editor/editor-switcher';
import { CloseButton, EditorTabs } from '../editor/editor-tabs';
import { EditorTextArea } from '../editor/editor-textarea';
import { CatchErrors } from '../react-errors';
import { CytoEditorHandlers } from './handlers';
import { FinderPopover, useCytoEditor } from './hooks';
import { useSpaceId } from '../hooks';
import { ChevronDownIcon, ChevronLeftIcon, SmallAddIcon, SmallCloseIcon } from '@chakra-ui/icons';
import EmojiPicker from 'emoji-picker-react';
const MonacoEditor = dynamic(() => import('@monaco-editor/react').then(m => m.default), { ssr: false });
export function useEditorValueSaver(tab) {
    const [values, setValues] = useLocalStore('editor-values', {});
    const tempValueRef = useRef();
    const valuesRef = useRef();
    valuesRef.current = values;
    const setValuesDebounced = useDebounceCallback((value) => {
        setValues(value);
    }, 500);
    return {
        valuesRef,
        tempValueRef,
        value: values[tab],
        setValue: useCallback((id, value) => {
            tempValueRef.current = Object.assign(Object.assign({}, valuesRef.current), { [id]: value });
            setValuesDebounced(Object.assign(Object.assign({}, valuesRef.current), { [id]: value }));
        }, []),
    };
}
export function useEditorTabs() {
    const [tabs, setTabs] = useLocalStore('editor-tabs', []);
    const [tab, setTab] = useLocalStore('editor-tab', 0);
    const tabsRef = useRef();
    tabsRef.current = tabs;
    return {
        tabs,
        addTab: useCallback((tab) => {
            if (!tabsRef.current.find((t) => t.id === tab.id)) {
                setTabs([...tabsRef.current, tab]);
            }
        }, []),
        closeTab: useCallback((id) => {
            const newTabs = tabsRef.current.filter((tab) => tab.id !== id);
            setTabs(newTabs);
            if (newTabs.length)
                setTab(newTabs[newTabs.length - 1].id);
            else
                setTab(0);
        }, []),
        tabId: tab,
        tab: tabs.find((t) => +t.id === +tab),
        activeTab: useCallback((id) => {
            setTab(id);
        }, []),
        setTab: useCallback((tab) => {
            setTabs(tabsRef.current.map((t) => (t.id === tab.id ? tab : t)));
        }, []),
        setTabs,
    };
}
export function useFindClientHandlerByCode({ codeLinkId, }) {
    const deep = useDeep();
    const [hid, setHid] = useState();
    const prevCodeLinkId = useRef();
    useEffect(() => {
        (() => __awaiter(this, void 0, void 0, function* () {
            if (!codeLinkId || codeLinkId === prevCodeLinkId.current)
                return;
            const { data: handlers } = yield deep.select({
                execution_provider_id: { _eq: deep.idLocal('@deep-foundation/core', 'JSExecutionProvider'), },
                isolation_provider_id: { _eq: deep.idLocal('@deep-foundation/core', 'ClientJSIsolationProvider'), },
                _or: [
                    { dist_id: { _eq: codeLinkId } },
                    { src_id: { _eq: codeLinkId } },
                ],
            }, { table: 'handlers', returning: 'handler_id dist_id src_id' });
            setHid(handlers === null || handlers === void 0 ? void 0 : handlers[0]);
            prevCodeLinkId.current = codeLinkId;
        }))();
    }, [codeLinkId, hid]);
    return hid;
}
export const List = React.memo(function List({ link }) {
    const deep = useDeep();
    const { data: contained } = deep.useDeepSubscription({ in: { type_id: deep.idLocal('@deep-foundation/core', 'Contain'), from_id: link.id } });
    return _jsx(_Fragment, { children: _jsx(Box, { sx: { paddingLeft: 4 }, children: contained.map(c => _jsx(Item, { link: c }, c.id)) }) });
}, () => true);
export const Item = React.memo(function Item({ link, openable = false, deletable = false, children = null, portalRef, closeTab, activeTab, addTab, isActive = false, }) {
    var _a, _b, _c;
    const [opened, setOpened] = useState(false);
    const [spaceId] = useSpaceId();
    const deep = useDeep();
    const [currentSymbolLink] = deep.useMinilinksSubscription({ type_id: deep.idLocal('@deep-foundation/core', 'Symbol'), to_id: link.id });
    const [typeSymbolLink] = deep.useMinilinksSubscription({ type_id: deep.idLocal('@deep-foundation/core', 'Symbol'), to_id: link.type_id });
    const currentSymbol = (_a = currentSymbolLink === null || currentSymbolLink === void 0 ? void 0 : currentSymbolLink.value) === null || _a === void 0 ? void 0 : _a.value;
    const typeSymbol = (_b = typeSymbolLink === null || typeSymbolLink === void 0 ? void 0 : typeSymbolLink.value) === null || _b === void 0 ? void 0 : _b.value;
    return _jsxs(_Fragment, { children: [_jsxs(Button, { sx: { position: 'relative', display: 'block' }, width: "100%", textAlign: "left", isActive: isActive, size: "sm", p: 1, variant: "ghost", onDoubleClick: () => {
                }, onClick: (event) => {
                    var _a, _b;
                    if ((event === null || event === void 0 ? void 0 : event.target) == (event === null || event === void 0 ? void 0 : event.currentTarget) || ((_a = event === null || event === void 0 ? void 0 : event.target) === null || _a === void 0 ? void 0 : _a.type) != 'button') {
                        addTab({
                            id: link.id, saved: true,
                            initialValue: deep.stringify((_b = link === null || link === void 0 ? void 0 : link.value) === null || _b === void 0 ? void 0 : _b.value),
                        });
                        activeTab(link.id);
                    }
                }, children: [_jsxs(Box, { children: [!!currentSymbol && _jsx(_Fragment, { children: _jsx(IconButton, { variant: 'outline', size: "xs", icon: _jsx(_Fragment, { children: currentSymbol }), "aria-label": 'icon change', onClick: () => __awaiter(this, void 0, void 0, function* () {
                                        yield deep.delete({ _or: [
                                                { type_id: deep.idLocal('@deep-foundation/core', 'Contain'), from_id: spaceId, to: { type_id: deep.idLocal('@deep-foundation/core', 'Symbol'), from_id: link.id } },
                                                { type_id: deep.idLocal('@deep-foundation/core', 'Symbol'), from_id: link.id },
                                            ], });
                                    }) }) }), !currentSymbol && _jsx(_Fragment, { children: _jsxs(Popover, { isLazy: true, children: [_jsx(PopoverTrigger, { children: _jsx(IconButton, { variant: 'solid', size: "xs", icon: _jsx(_Fragment, { children: typeSymbol || '' }), "aria-label": 'icon change' }) }), _jsx(Portal, { containerRef: portalRef, children: _jsx(PopoverContent, { children: _jsx(EmojiPicker, { onEmojiClick: (_d) => __awaiter(this, [_d], void 0, function* ({ emoji }) {
                                                        var _e, _f, _g, _h, _j, _k;
                                                        if (!((_g = (_f = (_e = link === null || link === void 0 ? void 0 : link.inByType) === null || _e === void 0 ? void 0 : _e[deep.idLocal('@deep-foundation/core', 'Symbol')]) === null || _f === void 0 ? void 0 : _f[0]) === null || _g === void 0 ? void 0 : _g.id)) {
                                                            yield deep.insert({
                                                                type_id: deep.idLocal('@deep-foundation/core', 'Symbol'), to_id: link.id, from_id: link.id,
                                                                string: { data: { value: emoji } },
                                                                in: { data: { type_id: deep.idLocal('@deep-foundation/core', 'Contain'), from_id: spaceId } },
                                                            });
                                                        }
                                                        else {
                                                            yield deep.update({ link_id: (_k = (_j = (_h = link === null || link === void 0 ? void 0 : link.inByType) === null || _h === void 0 ? void 0 : _h[deep.idLocal('@deep-foundation/core', 'Symbol')]) === null || _j === void 0 ? void 0 : _j[0]) === null || _k === void 0 ? void 0 : _k.id }, { value: emoji }, { table: 'strings' });
                                                        }
                                                    }) }) }) })] }) }), "\u00A0", _jsxs(Editable, { selectAllOnFocus: true, defaultValue: `${deep.nameLocal(link.id) == `${link.id}` ? '' : deep.nameLocal(link.id)}`, placeholder: link.type_id === deep.idLocal('@deep-foundation/core', 'Package') ? (_c = link === null || link === void 0 ? void 0 : link.value) === null || _c === void 0 ? void 0 : _c.value : link.id, display: "inline", onSubmit: (value) => __awaiter(this, void 0, void 0, function* () {
                                    var _l, _m, _o, _p;
                                    if ((_m = (_l = deep.minilinks.byId[link.id].inByType[deep.idLocal('@deep-foundation/core', 'Contain')]) === null || _l === void 0 ? void 0 : _l[0]) === null || _m === void 0 ? void 0 : _m.value) {
                                        yield deep.update({ link: { type_id: deep.idLocal('@deep-foundation/core', 'Contain'), to_id: link.id } }, { value: value }, { table: 'strings' });
                                    }
                                    else {
                                        yield deep.insert({ link_id: (_p = (_o = deep.minilinks.byId[link.id].inByType[deep.idLocal('@deep-foundation/core', 'Contain')]) === null || _o === void 0 ? void 0 : _o[0]) === null || _p === void 0 ? void 0 : _p.id, value: value }, { table: 'strings' });
                                    }
                                }), onCancel: (value) => { }, children: [_jsx(EditablePreview, { w: "calc(100% - 32px)" }), _jsx(EditableInput, { w: "calc(100% - 32px)" })] })] }), _jsxs(Box, { position: 'absolute', right: 1, top: 1, children: [children, deletable && _jsx(IconButton, { isRound: true, "aria-label": 'open level', fontSize: '20px', size: 'xs', onClick: () => __awaiter(this, void 0, void 0, function* () {
                                    if (confirm(`Delete ${link.id} ${deep.nameLocal(link.id)} (${deep.nameLocal(link.type_id)})`)) {
                                        yield deep.delete({ _or: [
                                                { type_id: deep.idLocal('@deep-foundation/core', 'Contain'), from_id: spaceId, to_id: link.id },
                                                { id: link.id },
                                            ], });
                                        closeTab(link.id);
                                    }
                                }), icon: _jsx(SmallCloseIcon, {}) }), "\u00A0", openable && _jsx(IconButton, { isRound: true, "aria-label": 'open level', fontSize: '20px', size: 'xs', onClick: () => setOpened(o => !o), icon: opened ? _jsx(ChevronDownIcon, {}) : _jsx(ChevronLeftIcon, {}) })] })] }), opened && _jsx(List, { link: link })] });
});
export const CytoEditorNav = React.memo(function CytoEditorNav({ portalRef }) {
    const deep = useDeep();
    const [spaceId] = useSpaceId();
    const [space] = deep.useMinilinksSubscription({ id: spaceId || 0 });
    const links = deep.useMinilinksSubscription({ in: { type_id: deep.idLocal('@deep-foundation/core', 'Contain'), from_id: spaceId || 0 } });
    const { tab, tabs, setTab, activeTab, tabId, setTabs, addTab, closeTab, } = useEditorTabs();
    const PortalProps = useMemo(() => ({
        containerRef: portalRef,
    }), []);
    return _jsx(_Fragment, { children: _jsxs("div", { style: { position: 'absolute', left: 0, top: 0, width: 300, height: '100%', overflowY: 'scroll' }, children: [!!space && _jsx(Item, { link: space, portalRef: portalRef, closeTab: closeTab, activeTab: activeTab, addTab: addTab, isActive: +(tab === null || tab === void 0 ? void 0 : tab.id) === +spaceId, children: _jsx(FinderPopover, { link: space, PortalProps: PortalProps, search: '', onSubmit: (link) => __awaiter(this, void 0, void 0, function* () {
                            yield deep.insert({
                                type_id: deep.idLocal('@deep-foundation/core', 'Contain'),
                                string: { data: { value: '' } },
                                from_id: spaceId,
                                to: { data: { type_id: link.id } },
                            });
                        }), children: _jsx(IconButton, { isRound: true, "aria-label": 'open level', fontSize: '20px', size: 'xs', icon: _jsx(_Fragment, { children: _jsx(SmallAddIcon, {}) }) }) }) }), _jsx(Divider, {}), links.map(l => _jsx(Item, { openable: true, deletable: true, link: l, portalRef: portalRef, closeTab: closeTab, activeTab: activeTab, addTab: addTab, isActive: +(tab === null || tab === void 0 ? void 0 : tab.id) === +l.id }, l.id))] }) });
}, () => true);
export const CytoEditor = React.memo(function CytoEditor() {
    var _a, _b, _c, _d, _e, _f, _g;
    const [cytoEditor, setCytoEditor] = useCytoEditor();
    const onClose = useCallback(() => {
        setCytoEditor(false);
    }, []);
    const deep = useDeep();
    const { tab, tabs, closeTab, setTab, activeTab, tabId, setTabs, } = useEditorTabs();
    const portalRef = useRef();
    const [currentLinkId, setCurrentLinkId] = useState(tabId || 0);
    if (!currentLinkId && tabId) {
        setCurrentLinkId(tabId);
    }
    const { data: [currentLink = (_a = deep === null || deep === void 0 ? void 0 : deep.minilinks) === null || _a === void 0 ? void 0 : _a.byId[currentLinkId]] = [] } = useDeepSubscription({
        id: currentLinkId || 0,
    });
    const onCloseAll = useCallback(() => {
        setTabs([]);
    }, []);
    const { tempValueRef, valuesRef, value, setValue, } = useEditorValueSaver(tabId);
    const hid = useFindClientHandlerByCode({
        codeLinkId: tabId,
    });
    const handlerId = hid === null || hid === void 0 ? void 0 : hid.handler_id;
    const generatedLink = useMinilinksFilter(deep.minilinks, (link) => {
        var _a, _b, _c;
        const filterResult = ((_c = (_b = (_a = link === null || link === void 0 ? void 0 : link.outByType) === null || _a === void 0 ? void 0 : _a[deep.idLocal('@deep-foundation/core', 'GeneratedFrom')]) === null || _b === void 0 ? void 0 : _b[0]) === null || _c === void 0 ? void 0 : _c.to_id) === tabId;
        return filterResult;
    }, (link, ml) => {
        var _a, _b, _c;
        return (_c = (_b = (_a = ml === null || ml === void 0 ? void 0 : ml.byId[tabId]) === null || _a === void 0 ? void 0 : _a.inByType[deep.idLocal('@deep-foundation/core', 'GeneratedFrom')]) === null || _b === void 0 ? void 0 : _b[0]) === null || _c === void 0 ? void 0 : _c.from;
    });
    const [currentLanguage, setCurrentLanguage] = useLocalStore('df-dc-editor-currentLanguage', 'plaintext');
    useEffect(() => {
        var _a, _b;
        (_a = refEditor.current) === null || _a === void 0 ? void 0 : _a.monaco.editor.setModelLanguage((_b = refEditor.current) === null || _b === void 0 ? void 0 : _b.monaco.editor.getModels()[0], currentLanguage);
    }, [currentLanguage]);
    const currentValue = ((_b = valuesRef === null || valuesRef === void 0 ? void 0 : valuesRef.current) === null || _b === void 0 ? void 0 : _b[tabId]) || typeof (tab === null || tab === void 0 ? void 0 : tab.initialValue) !== 'undefined' ? tab === null || tab === void 0 ? void 0 : tab.initialValue : '';
    const refEditor = useRef();
    const [rightArea, setRightArea] = useState('preview');
    const [fillSize, setFillSize] = useState(false);
    const [viewSize, setViewSize] = useState({ width: '50%', height: '100%' });
    const [editorMounted, setEditorMounted] = useState(false);
    const errorRenderer = useMemo(() => {
        return (error, reset) => {
            return _jsx("div", { children: json5.stringify(error) });
        };
    }, []);
    const focusEditor = useCallback(() => {
        var _a, _b;
        (_b = (_a = refEditor === null || refEditor === void 0 ? void 0 : refEditor.current) === null || _a === void 0 ? void 0 : _a.editor) === null || _b === void 0 ? void 0 : _b.focus();
    }, []);
    const languages = (_c = refEditor.current) === null || _c === void 0 ? void 0 : _c.monaco.languages.getLanguages();
    const validationTS = (_d = refEditor.current) === null || _d === void 0 ? void 0 : _d.monaco.languages.typescript.typescriptDefaults.setDiagnosticsOptions({
        noSemanticValidation: true,
        noSyntaxValidation: true,
    });
    const { colorMode } = useColorMode();
    const [Component, setComponent] = useState(undefined);
    useEffect(() => {
        var _a, _b;
        if (handlerId) {
            return;
        }
        const value = ((_a = generatedLink === null || generatedLink === void 0 ? void 0 : generatedLink.value) === null || _a === void 0 ? void 0 : _a.value) || ((_b = currentLink === null || currentLink === void 0 ? void 0 : currentLink.value) === null || _b === void 0 ? void 0 : _b.value);
        console.log('editor', 'evalClientHandler', 'useEffect', value);
        if (!value) {
            return;
        }
        evalClientHandler({ value, deep }).then(({ data, error }) => {
            console.log('editor', 'evalClientHandler', 'error', error);
            console.log('editor', 'evalClientHandler', 'data', data);
            if (!error && data) {
                setComponent(() => data);
            }
            else if (Component !== undefined) {
                setComponent(undefined);
            }
        });
    }, [handlerId, (_e = currentLink === null || currentLink === void 0 ? void 0 : currentLink.value) === null || _e === void 0 ? void 0 : _e.value, (_f = generatedLink === null || generatedLink === void 0 ? void 0 : generatedLink.value) === null || _f === void 0 ? void 0 : _f.value]);
    return _jsx(_Fragment, { children: _jsxs(Modal, { isOpen: cytoEditor, onClose: onClose, size: 'full', onEsc: onClose, children: [_jsx(ModalOverlay, {}), _jsxs(ModalContent, { sx: { height: '100vh', overflow: 'initial', position: 'relative' }, children: [_jsx("div", { ref: portalRef }), _jsx(EditorGrid, { sash: true, editorTextAreaElement: _jsx(_Fragment, { children: [
                                    _jsx(CytoEditorNav, { portalRef: portalRef }),
                                    _jsxs(Box, { sx: {
                                            pos: 'relative',
                                            left: 300,
                                            height: '100%',
                                            width: 'calc(100% - 300px)',
                                        }, children: [_jsx(EditorTextArea, { refEditor: refEditor, value: currentValue, defaultLanguage: currentLanguage, onChange: (value) => {
                                                    setValue(tabId, value);
                                                    setTab(Object.assign(Object.assign({}, tab), { saved: tab.initialValue === value }));
                                                }, onClose: () => {
                                                    var _a;
                                                    if (tabs.length === 1 && ((_a = tabs[0]) === null || _a === void 0 ? void 0 : _a.id) === tab.id)
                                                        onClose();
                                                    closeTab(tabId);
                                                    setValue(tabId, undefined);
                                                    focusEditor();
                                                }, onSave: (savedValue) => __awaiter(this, void 0, void 0, function* () {
                                                    var _h, _j;
                                                    const value = ((_h = tempValueRef === null || tempValueRef === void 0 ? void 0 : tempValueRef.current) === null || _h === void 0 ? void 0 : _h[tabId]) || savedValue;
                                                    const Value = yield deep.id({ in: { type_id: { _id: ['@deep-foundation/core', 'Value'] }, from: { typed: { id: { _eq: tab.id } } } } });
                                                    const table = Value === deep.idLocal('@deep-foundation/core', 'String') ? 'strings' : Value === deep.idLocal('@deep-foundation/core', 'Number') ? 'numbers' : Value === deep.idLocal('@deep-foundation/core', 'Object') ? 'objects' : undefined;
                                                    const type = Value === deep.idLocal('@deep-foundation/core', 'String') ? 'string' : Value === deep.idLocal('@deep-foundation/core', 'Number') ? 'number' : Value === deep.idLocal('@deep-foundation/core', 'Object') ? 'object' : 'undefined';
                                                    let _value;
                                                    try {
                                                        _value = table === 'strings' ? value : table === 'numbers' ? parseFloat(value) : table === 'objects' ? json5.parse(value) : undefined;
                                                    }
                                                    catch (error) {
                                                        console.log('error123', error);
                                                    }
                                                    if (!((_j = deep.minilinks.byId[tab.id]) === null || _j === void 0 ? void 0 : _j.value)) {
                                                        yield deep.insert({ link_id: tab.id, value: _value }, {
                                                            table: table,
                                                        });
                                                        setTab(Object.assign(Object.assign({}, tab), { initialValue: value, loading: false, saved: true }));
                                                    }
                                                    else if (type !== 'undefined') {
                                                        yield deep.update({ link_id: { _eq: tab.id } }, {
                                                            value: _value,
                                                        }, {
                                                            table: `${type}s`,
                                                        });
                                                        setTab(Object.assign(Object.assign({}, tab), { initialValue: value, loading: false, saved: true }));
                                                    }
                                                    else {
                                                        setTab(Object.assign(Object.assign({}, tab), { initialValue: value, loading: false, saved: false }));
                                                    }
                                                }), onMount: () => setEditorMounted(true) }), _jsx(Box, { w: '100%', pos: 'absolute', bottom: '0', borderTopColor: 'borderColor', borderTopWidth: 'thin', p: '0.5rem', height: 'auto', bg: 'lightDark', children: _jsx(Box, { pos: 'relative', height: '100%', children: _jsx(ListLanguages, { languages: languages, currentLanguage: currentLanguage, setLanguage: (i) => {
                                                            if (i == 'typescript')
                                                                validationTS;
                                                            setCurrentLanguage(i);
                                                        } }) }) })] }, tabId)
                                ] }), editorTabsElement: _jsx(EditorTabs, { tabs: tabs.map((tab) => (Object.assign(Object.assign({}, tab), { active: tabId === tab.id }))), setTabs: (tabs) => setTabs(tabs), onClose: (tab) => {
                                    var _a;
                                    if (tabs.length === 1 && ((_a = tabs[0]) === null || _a === void 0 ? void 0 : _a.id) === tab.id)
                                        onClose();
                                    closeTab(tab.id);
                                    setValue(tabId, undefined);
                                    focusEditor();
                                }, onClick: (tab) => {
                                    activeTab(tab.id);
                                    focusEditor();
                                }, onSaveTab: (savedValue) => __awaiter(this, void 0, void 0, function* () {
                                    var _k, _l;
                                    const value = ((_k = tempValueRef === null || tempValueRef === void 0 ? void 0 : tempValueRef.current) === null || _k === void 0 ? void 0 : _k[tabId]) || savedValue;
                                    const Value = yield deep.id({ in: { type_id: { _id: ['@deep-foundation/core', 'Value'] }, from: { typed: { id: { _eq: tab.id } } } } });
                                    const table = Value === deep.idLocal('@deep-foundation/core', 'String') ? 'strings' : Value === deep.idLocal('@deep-foundation/core', 'Number') ? 'numbers' : Value === deep.idLocal('@deep-foundation/core', 'Object') ? 'objects' : undefined;
                                    const type = Value === deep.idLocal('@deep-foundation/core', 'String') ? 'string' : Value === deep.idLocal('@deep-foundation/core', 'Number') ? 'number' : Value === deep.idLocal('@deep-foundation/core', 'Object') ? 'object' : 'undefined';
                                    let _value;
                                    try {
                                        _value = table === 'strings' ? value : table === 'numbers' ? parseFloat(value) : table === 'objects' ? json5.parse(value) : undefined;
                                    }
                                    catch (error) { }
                                    if (!((_l = deep.minilinks.byId[tab.id]) === null || _l === void 0 ? void 0 : _l.value)) {
                                        yield deep.insert({ link_id: tab.id, value: _value }, {
                                            table: table,
                                        });
                                        setTab(Object.assign(Object.assign({}, tab), { initialValue: value, loading: false, saved: true }));
                                    }
                                    else if (type !== 'undefined') {
                                        yield deep.update({ link_id: { _eq: tab.id } }, {
                                            value: _value,
                                        }, {
                                            table: `${type}s`,
                                        });
                                        setTab(Object.assign(Object.assign({}, tab), { initialValue: value, loading: false, saved: true }));
                                    }
                                    else {
                                        setTab(Object.assign(Object.assign({}, tab), { initialValue: value, loading: false, saved: false }));
                                    }
                                    console.log('onclick');
                                }) }), closeAllButtonElement: _jsx(IconButton, { icon: _jsx(VscClearAll, {}), onClick: onCloseAll, "aria-label": 'Close all tabs' }), closeButtonElement: _jsx(CloseButton, { onClick: onClose }), editorRight: rightArea === 'handlers' && (_jsx(CytoEditorHandlers, { linkId: tab === null || tab === void 0 ? void 0 : tab.id, handleredableIds: generatedLink ? [generatedLink === null || generatedLink === void 0 ? void 0 : generatedLink.id, tab.id] : [tab === null || tab === void 0 ? void 0 : tab.id] })) ||
                                rightArea === 'preview' && _jsx(Box, { pos: 'relative', sx: {
                                        backgroundColor: 'editorPreviewBackground',
                                        backgroundImage: `linear-gradient(-90deg, ${'editorPreviewBackgroundGrid'} 1px, transparent 1px), linear-gradient(0deg, ${'editorPreviewBackgroundGrid'} 1px, transparent 1px), linear-gradient(transparent 0px, ${'editorPreviewBackground'} 1px, ${'editorPreviewBackground'} 20px, transparent 20px), linear-gradient(-90deg, ${'editorPreviewBackgroundGrid'} 1px, transparent 1px), linear-gradient(-90deg, transparent 0px, ${'editorPreviewBackground'} 1px, ${'editorPreviewBackground'} 20px, transparent 20px), linear-gradient(0deg, ${'editorPreviewBackgroundGrid'} 1px, transparent 1px)`,
                                        backgroundSize: '20px 20px, 20px 20px, 20px 20px, 20px 20px, 20px 20px, 20px 20px',
                                    }, children: [_jsxs(EditorComponentView, { size: viewSize, onChangeSize: (viewSize) => setViewSize(viewSize), fillSize: fillSize, setFillSize: setFillSize, children: [handlerId && [_jsx(ClientHandler, { handlerId: handlerId, fillSize: fillSize, linkId: currentLink === null || currentLink === void 0 ? void 0 : currentLink.id, ml: deep.minilinks }, `${(currentLink === null || currentLink === void 0 ? void 0 : currentLink.id) || 0}-${tabId}-${handlerId}`)], !handlerId && typeof (Component) === 'function' && [_jsx(CatchErrors, { errorRenderer: (error) => {
                                                            console.log('EditorComponentView', 'errorRenderer', error);
                                                            return _jsx("div", { children: JSON.stringify(error) });
                                                        }, children: _jsx(ClientHandlerRenderer, { Component: Component, fillSize: fillSize, link: (_g = deep === null || deep === void 0 ? void 0 : deep.minilinks) === null || _g === void 0 ? void 0 : _g.byId[currentLink === null || currentLink === void 0 ? void 0 : currentLink.id] }) }, Component.toString())]] }, `${(currentLink === null || currentLink === void 0 ? void 0 : currentLink.id) || 0}-${tabId}-${handlerId}`)] }) ||
                                rightArea === 'results' && _jsx(EditorResults, { id: tab.id }), editorRightSwitch: _jsx(EditorSwitcher, { portalRef: portalRef, fillSize: fillSize, setFillSize: (newFillSize) => {
                                    setFillSize(newFillSize);
                                    if (!fillSize)
                                        setViewSize({ width: 250, height: 250 });
                                }, currentLinkId: currentLinkId, setCurrentLinkId: (newCurrentLinkId) => {
                                    setCurrentLinkId(newCurrentLinkId);
                                }, area: rightArea, setArea: (rightArea) => {
                                    setRightArea(rightArea);
                                } }) })] })] }) });
}, () => true);
//# sourceMappingURL=editor.js.map